#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libn64.h>
#include <malloc.h>
#include "audio.h"
#include "spc_codec.h"

#define XRES 320
#define YRES 250
unsigned short screen[XRES*YRES];

uint8_t spc_file[0x10180];

// execution begins here
int UltraMain(void) {
    struct VI_config video;
    int draw=0;
    static int row=3;

    // buffer for rendering text strings (only sprintf works at the moment)
    char buf[1024]; 

    // enable MI interrupts (on the CPU), but clear all specific device
    // interrupts
    set_MI_interrupt(1,1);

    // setup video
    video.bitdepth=16;
    video.xres=XRES;
    video.yres=YRES;
    video.xscale=0.5;
    video.yscale=0.25;

    video.framebuffer=UncachedAddr(screen);

    VI_FillScreen(&video,0);

    // commit video
    VI_WriteRegs(&video);

    //
    audio_init();

    // setup SPC
    spc_emu_start();

    data_cache_writeback_invalidate(spc_file,sizeof(spc_file));
    PI_read(spc_file,0x10200000,sizeof(spc_file));
    data_cache_invalidate(spc_file,sizeof(spc_file));
    const char *banner[] = {
"  SNEON64 (r66)",
"Based on Rockbox spc decoder,",
"based on Game_Music_Emu,",
"based on OpenSPC!",
"",
"Copyright (C) 2006-2010 Adam Gashlin",
"Copyright (C) 2007-2008 Michael Sevakis",
"Copyright (C) 2004-2007 Shay Green",
"Copyright (C) 2002 Brad Martin",
""};

    for (int i = 0; i < sizeof(banner)/sizeof(banner[0]); i++) {
        VI_DrawText(&video,16,row++*8,banner[i]);
    }

    if (load_spc_buffer(spc_file,sizeof(spc_file))) {
        sprintf(buf,"load success!");
    } else {
        sprintf(buf,"load failure");
    }

    VI_DrawText(&video,16,row++*8,buf);

    while (1) {
        short * sb;
        int sbsamples;

        unsigned long starttime,endtime;
        unsigned long audiostarttime,audioendtime;

        static unsigned long totaltime = 1;
        static unsigned long audiotime = 0;

        starttime = read_count();

        // get audio properties
        sb=audio_getbuf();
        sbsamples=audio_getbufsamplecount();

        audiostarttime = read_count();
        // generate audio

        // TODO
        spc_play_get_samples(sb);
        audioendtime = read_count();

        // queue for playback
        audio_done();

        if (draw) {
            /* tenths of milliseconds */
            unsigned long atime = audiotime*10000ULL/COUNTS_PER_SECOND;
            unsigned long ttime = totaltime*10000ULL/COUNTS_PER_SECOND;
            unsigned long btime = AUDIO_BUFFER_SAMPLES*10000ULL/AUDIO_FREQUENCY;
            sprintf(buf,"a%2lu.%lums t%2lu.%lums b%2lu.%lums"
#ifdef AUDIO_DEBUG
                   " d%d r%d"
#endif
                    ,atime/10,atime%10,ttime/10,ttime%10,btime/10,btime%10
#ifdef AUDIO_DEBUG
                    ,audio_buffers_deferred,audio_restarts
#endif
                   );
            VI_DrawText(&video,16,row++*8,buf);
        }

        if (row > 27) {
            draw=1;
            row = 3;
            int i;
            int redrawcount;
            redrawcount = 1;
            for (i=0;i<redrawcount;i++)
                VI_FillScreen(&video,0);
        }

        endtime = read_count();

        totaltime = endtime-starttime;
        audiotime = audioendtime-audiostarttime;
    }

    return 0;
}

